<div class="panel nice-padding panel-upgrade-notification">
	<div class="help-block help-warning">
		You have not entered your Google Analytics code.
		<a href="{{ url("wagtailsettings:edit", 'coop', 'analyticssettings', request.site.pk) }}">
			Set up Google Analytics</a>.
	</div>
</div>
